

import random
import logging

from thanatos import categories
from thanatos.database import inventory
from thanatos.questions.base import Question

_log = logging.getLogger('thanatos')


class SlotsQuestion(Question):
    """ Asks what region boards another given region. """

    name = 'Slots Question'
    description = 'Asks how many high/mid/low/turret/launcher slots a given ship has.'
    category = categories.inventory
    sub_category = categories.inventory_slots

    random_weight = 20

    question = 'How many {} slots does the {} have?'

    high_slots_dogma_attribute = 14

    slots = {
        'high': 14,
        'mid': 13,
        'low': 12,
        'launcher': 101,
        'turret': 102,
    }

    slots_min = 0
    slots_max = 9  # 1 more than the max of 8 due to how range works

    def ask(self):
        all_ships = inventory.get_all_published_ships_basic(self.db_connection)
        chosen_slot = random.choice(self.slots.keys())

        # Go into a while loop until we get a ship that actually has a value for the chosen dogma attribute
        while True:
            chosen_ship = random.choice(all_ships)

            slot_count = inventory.get_dogma_attribute_for_type(
                self.db_connection,
                chosen_ship[0],
                self.slots[chosen_slot]
            )

            if slot_count is not None:
                correct_answer = (int(slot_count), int(slot_count))

                possible_answers = range(self.slots_min, self.slots_max)
                possible_wrong_answers = list(set(possible_answers) - set([correct_answer[0]]))
                possible_wrong_answers = [(x, x) for x in possible_wrong_answers]

                question = self.format_question(correct_answer, possible_wrong_answers, self.question.format(chosen_slot, chosen_ship[1]))

                return question


class ShipImageIdentificationQuestion(Question):
    """ Asks the user to select the correct ship based on an image server link. """

    name = 'Ship Image Identification'
    description = 'Pick what ship is shown in the provided image.'
    category = categories.inventory
    sub_category = categories.inventory_ship_id

    random_weight = 16

    question = 'What ship is pictured?'

    def ask(self):
        all_ships = inventory.get_all_published_ships_basic(self.db_connection)
        chosen_ship = random.choice(all_ships)

        possible_wrong_answers = list(set(all_ships) - set([chosen_ship]))
        possible_wrong_answers = [(x[0], x[1]) for x in possible_wrong_answers]

        question = self.format_question(chosen_ship, possible_wrong_answers, self.question, add_images_to_question=True)

        return question


class ShipImageIdentificationHardQuestion(Question):
    """ Asks the user to select the correct ship based on an image server link but
    the answers are all variations of the same hull. """

    name = 'Ship Image Identification (Hard)'
    description = 'Pick what ship is shown in the provided image.'
    category = categories.inventory
    sub_category = categories.inventory_ship_id

    random_weight = 8

    question = 'What ship is pictured?'

    def ask(self):
        ships_with_variations = inventory.get_ships_that_have_variations(self.db_connection)
        chosen_hull = random.choice(ships_with_variations)

        possible_ships = inventory.get_type_variations(self.db_connection, chosen_hull[0])
        chosen_ship = random.choice(possible_ships)

        possible_wrong_answers = list(set(possible_ships) - set([chosen_ship]))
        possible_wrong_answers = [(x[0], x[1]) for x in possible_wrong_answers]

        question = self.format_question(chosen_ship, possible_wrong_answers, self.question, add_images_to_question=True)

        return question





import random
import logging

from thanatos import categories, lore
from thanatos.questions.base import Question

_log = logging.getLogger('thanatos')


def get_random_weight(questions):
    weight = len(questions) / 4

    if weight > 1:
        return weight

    else:
        return 1


class CapsuleerLoreQuestion(Question):
    """ Asks questions about capsuleer created lore. """

    name = 'Capsuleer Lore'
    description = 'Asks questions about capsuleer created lore.'
    category = categories.lore
    sub_category = categories.lore_capsuleer

    random_weight = get_random_weight(lore.capsuleer_lore_questions)

    question = 'Capsuleer lore questions.'

    def ask(self):
        raw_question = random.choice(lore.capsuleer_lore_questions)

        correct_answer = (-1, raw_question['answer'])

        possible_wrong_answers = [(x, raw_question['wrong_answers'][x]) for x in xrange(0, len(raw_question['wrong_answers']))]

        question = self.format_question(correct_answer, possible_wrong_answers, raw_question['question'])

        return question


class NPCLoreQuestion(Question):
    """ Asks questions about capsuleer created lore. """

    name = 'NPC Lore'
    description = 'Asks questions about NPC created lore.'
    category = categories.lore
    sub_category = categories.lore_npc

    random_weight = get_random_weight(lore.npc_lore_questions)

    question = 'NPC lore questions.'

    def ask(self):
        raw_question = random.choice(lore.npc_lore_questions)

        correct_answer = (-1, raw_question['answer'])

        possible_wrong_answers = [(x, raw_question['wrong_answers'][x]) for x in xrange(0, len(raw_question['wrong_answers']))]

        question = self.format_question(correct_answer, possible_wrong_answers, raw_question['question'])

        return question





import mock
import MySQLdb
import unittest2

from thanatos.questions import inventory


class SlotsQuestionTestCase(unittest2.TestCase):

    def setUp(self):
        self.mock_db_connection = mock.Mock(spec=MySQLdb.connection)

    def test_class_initializes(self):
        """ Simply test we can create an instance of the class. """

        inventory.SlotsQuestion(self.mock_db_connection)

    @mock.patch('thanatos.questions.inventory.inventory')
    @mock.patch('thanatos.questions.base.Question.format_question')
    @mock.patch('random.choice')
    def test_question_ask(self, mock_random_choice, mock_format_question, mock_db_inventory):
        """ Test we can call the slots question ask method. """

        mock_db_inventory.get_all_published_ships_basic.return_value = [(582L, 'Bantam', 25L, 'Frigate', 6L, 'Ship')]
        mock_db_inventory.get_dogma_attribute_for_type.side_effect = [
            None,
            3,
        ]

        mock_random_choice.side_effect = [
            'high',
            (582L, 'Bantam', 25L, 'Frigate', 6L, 'Ship'),
            (582L, 'Bantam', 25L, 'Frigate', 6L, 'Ship'),
        ]

        inventory.SlotsQuestion(self.mock_db_connection).ask()

        mock_format_question.assert_called_with(
            (3, 3),
            [(0, 0), (1, 1), (2, 2), (4, 4), (5, 5), (6, 6), (7, 7), (8, 8)],
            'How many high slots does the Bantam have?',
        )


class ShipIDQuestionTestCase(unittest2.TestCase):

    def setUp(self):
        self.mock_db_connection = mock.Mock(spec=MySQLdb.connection)

    def test_class_initializes(self):
        """ Simply test we can create an instance of the class. """

        inventory.ShipImageIdentificationQuestion(self.mock_db_connection)

    @mock.patch('thanatos.questions.inventory.inventory')
    @mock.patch('thanatos.questions.base.Question.format_question')
    @mock.patch('random.choice')
    def test_question_ask(self, mock_random_choice, mock_format_question, mock_db_inventory):
        """ Test we can call the ship identification question ask method. """

        mock_db_inventory.get_all_published_ships_basic.return_value = [
            (582L, 'Bantam', 25L, 'Frigate', 6L, 'Ship'),
            (583L, 'Tristin', 25L, 'Frigate', 6L, 'Ship'),
            (584L, 'Atron', 25L, 'Frigate', 6L, 'Ship'),
        ]

        mock_random_choice.return_value = (582L, 'Bantam', 25L, 'Frigate', 6L, 'Ship')

        inventory.ShipImageIdentificationQuestion(self.mock_db_connection).ask()

        mock_format_question.assert_called_with(
            (582L, 'Bantam', 25L, 'Frigate', 6L, 'Ship'),
            [(583L, 'Tristin'), (584L, 'Atron')],
            'What ship is pictured?',
            add_images_to_question=True
        )




import mock
import unittest2

from thanatos.questions import question_utils


class ThanatosUtilsTestCase(unittest2.TestCase):

    def setUp(self):
        pass
    
    def test_get_question_subclasses(self):
        question_utils.get_question_subclasses()




import mock
import unittest2

from thanatos.database import universe


class DatabaseUniverseTestCase(unittest2.TestCase):

    def setUp(self):
        pass

    @mock.patch('thanatos.database.universe.execute_sql')
    def test_get_all_regions(self, mock_execute_sql):
        """  """

        mock_db_connection = mock.MagicMock()
        mock_execute_sql.return_value = [(1, 'test')]

        results = universe.get_all_regions(mock_db_connection)

        mock_execute_sql.assert_called_with('CALL get_all_regions();', mock_db_connection)
        self.assertEqual(results, [(1, 'test')])
        self.assertEqual(universe.get_all_regions._results, [(1, 'test')])

    @mock.patch('thanatos.database.universe.execute_sql')
    def test_get_all_not_wh_regions(self, mock_execute_sql):
        """  """

        mock_db_connection = mock.MagicMock()
        mock_execute_sql.return_value = [(1, 'test')]

        results = universe.get_all_not_wh_regions(mock_db_connection)

        mock_execute_sql.assert_called_with('CALL get_all_not_wh_regions();', mock_db_connection)
        self.assertEqual(results, [(1, 'test')])
        self.assertEqual(universe.get_all_not_wh_regions._results, [(1, 'test')])

    @mock.patch('thanatos.database.universe.execute_sql')
    def test_get_all_regions_connected_to_region(self, mock_execute_sql):
        """  """

        mock_db_connection = mock.MagicMock()
        mock_region_id = 101
        mock_execute_sql.return_value = [(1, 'test')]

        results = universe.get_all_regions_connected_to_region(mock_db_connection, mock_region_id)

        mock_execute_sql.assert_called_with('CALL get_all_regions_connected_to_region(101);', mock_db_connection)
        self.assertEqual(results, [(1, 'test')])




capsuleer_lore_questions = [
    {
        'question': 'What was the name of the first capsuleer flown Titan class ship to be destroyed?',
        'answer': 'Steve',
        'wrong_answers': [
            'Steven',
            'Andie',
            'Scott',
            'Cyvok\'s Avatar',
        ]
    },
    {
        'question': 'How many Imperial Apocalypse Battleships were given out in the YC105 (2003) Amarr championship?',
        'answer': '5',
        'wrong_answers': [
            '3',
            '7',
            '9',
            '10',
        ]
    },
    {
        'question': 'What was the original name for the Band of Brothers (BoB) alliance?',
        'answer': 'Cookies, Cake, Cream and Pie (CCCP)',
        'wrong_answers': [
            'Cult of War (COW)',
            'Ascendant Frontier (ASCN)',
            'KenZoku (RKZ)',
            'Imperial Republic of the North (IRON)',
        ]
    },
    {
        'question': 'The Adrestia and Utu were awarded as prizes for which Alliance Tournament?',
        'answer': 'AT VIII',
        'wrong_answers': [
            'AT I',
            'AT II',
            'AT III',
            'AT IV',
            'AT V',
            'AT VI',
            'AT VII',
            'AT IX',
            'AT X',
            'AT XI',
            'AT XII',
        ]
    },
    {
        'question': 'The Vangel and Malice were awarded as prizes for which Alliance Tournament?',
        'answer': 'AT IX',
        'wrong_answers': [
            'AT I',
            'AT II',
            'AT III',
            'AT IV',
            'AT V',
            'AT VI',
            'AT VII',
            'AT VIII',
            'AT X',
            'AT XI',
            'AT XII',
        ]
    },
    {
        'question': 'The Etana and Cambion were awarded as prizes for which Alliance Tournament?',
        'answer': 'AT X',
        'wrong_answers': [
            'AT I',
            'AT II',
            'AT III',
            'AT IV',
            'AT V',
            'AT VI',
            'AT VII',
            'AT VIII',
            'AT IX',
            'AT XI',
            'AT XII',
        ]
    },
    {
        'question': 'The Moracha and Chremoas were awarded as prizes for which Alliance Tournament?',
        'answer': 'AT XI',
        'wrong_answers': [
            'AT I',
            'AT II',
            'AT III',
            'AT IV',
            'AT V',
            'AT VI',
            'AT VII',
            'AT VIII',
            'AT IX',
            'AT X',
            'AT XII',
        ]
    },
    {
        'question': 'The Whiptail and Chameleon were awarded as prizes for which Alliance Tournament?',
        'answer': 'AT XII',
        'wrong_answers': [
            'AT I',
            'AT II',
            'AT III',
            'AT IV',
            'AT V',
            'AT VI',
            'AT VII',
            'AT VIII',
            'AT IX',
            'AT X',
            'AT XI',
        ]
    },
    {
        'question': 'Who won Alliance Tournament I?',
        'answer': 'Band of Brothers',
        'wrong_answers': [
            'Agony Empire',
            'Ev0ke',
            'Hun Reloaded',
            'Hydra Reloaded',
            'Outbreak',
            'Pandemic Legion',
            'Suddenly Spaceships.',
            'Verge of Collapse',
            'The Camel Empire',
            'Pandemic Empire',
            'Legion of Brothers',
        ]
    },
    {
        'question': 'Who won Alliance Tournament II?',
        'answer': 'Band of Brothers',
        'wrong_answers': [
            'Agony Empire',
            'Ev0ke',
            'Hun Reloaded',
            'Hydra Reloaded',
            'Outbreak',
            'Pandemic Legion',
            'Suddenly Spaceships.',
            'Verge of Collapse',
            'The Camel Empire',
        ]
    },
    {
        'question': 'Who won Alliance Tournament III?',
        'answer': 'Band of Brothers',
        'wrong_answers': [
            'Agony Empire',
            'Ev0ke',
            'Hun Reloaded',
            'Hydra Reloaded',
            'Outbreak',
            'Pandemic Legion',
            'Suddenly Spaceships.',
            'Verge of Collapse',
            'The Camel Empire',
        ]
    },
    {
        'question': 'Who won Alliance Tournament IV?',
        'answer': 'Hun Reloaded',
        'wrong_answers': [
            'Agony Empire',
            'Band of Brothers',
            'Ev0ke',
            'Hydra Reloaded',
            'Outbreak',
            'Pandemic Legion',
            'Suddenly Spaceships.',
            'Verge of Collapse',
            'The Camel Empire',
        ]
    },
    {
        'question': 'Who won Alliance Tournament V?',
        'answer': 'Ev0ke',
        'wrong_answers': [
            'Agony Empire',
            'Band of Brothers',
            'Hun Reloaded',
            'Hydra Reloaded',
            'Outbreak',
            'Pandemic Legion',
            'Suddenly Spaceships.',
            'Verge of Collapse',
            'The Camel Empire',
        ]
    },
    {
        'question': 'Who won Alliance Tournament VI?',
        'answer': 'Pandemic Legion',
        'wrong_answers': [
            'Agony Empire',
            'Band of Brothers',
            'Ev0ke',
            'Hun Reloaded',
            'Hydra Reloaded',
            'Outbreak',
            'Suddenly Spaceships.',
            'Verge of Collapse',
            'The Camel Empire',
        ]
    },
    {
        'question': 'Who won Alliance Tournament VII?',
        'answer': 'Pandemic Legion',
        'wrong_answers': [
            'Agony Empire',
            'Band of Brothers',
            'Ev0ke',
            'Hun Reloaded',
            'Hydra Reloaded',
            'Outbreak',
            'Suddenly Spaceships.',
            'Verge of Collapse',
            'The Camel Empire',
        ]
    },
    {
        'question': 'Who won Alliance Tournament VIII?',
        'answer': 'Pandemic Legion',
        'wrong_answers': [
            'Agony Empire',
            'Band of Brothers',
            'Ev0ke',
            'Hun Reloaded',
            'Hydra Reloaded',
            'Outbreak',
            'Suddenly Spaceships.',
            'Verge of Collapse',
            'The Camel Empire',
        ]
    },
    {
        'question': 'Who won Alliance Tournament IX?',
        'answer': 'Hydra Reloaded',
        'wrong_answers': [
            'Agony Empire',
            'Band of Brothers',
            'Ev0ke',
            'Hun Reloaded',
            'Outbreak',
            'Pandemic Legion',
            'Suddenly Spaceships.',
            'Verge of Collapse',
            'The Camel Empire',
        ]
    },
    {
        'question': 'Who won Alliance Tournament X?',
        'answer': 'Verge of Collapse',
        'wrong_answers': [
            'Agony Empire',
            'Band of Brothers',
            'Ev0ke',
            'Hun Reloaded',
            'Hydra Reloaded',
            'Outbreak',
            'Pandemic Legion',
            'Suddenly Spaceships.',
            'The Camel Empire',
        ]
    },
    {
        'question': 'Who won Alliance Tournament XI?',
        'answer': 'Pandemic Legion',
        'wrong_answers': [
            'Agony Empire',
            'Band of Brothers',
            'Ev0ke',
            'Hun Reloaded',
            'Hydra Reloaded',
            'Outbreak',
            'Suddenly Spaceships.',
            'Verge of Collapse',
            'The Camel Empire',
        ]
    },
    {
        'question': 'Who won Alliance Tournament XII?',
        'answer': 'The Camel Empire',
        'wrong_answers': [
            'Agony Empire',
            'Band of Brothers',
            'Ev0ke',
            'Hun Reloaded',
            'Hydra Reloaded',
            'Outbreak',
            'Pandemic Legion',
            'Suddenly Spaceships.',
            'Verge of Collapse',
        ]
    },
    {
        'question': 'Who got banned from all future Alliance Tournaments in AT XII?',
        'answer': 'CODE.',
        'wrong_answers': [
            'Agony Empire',
            'Band of Brothers',
            'Ev0ke',
            'Hun Reloaded',
            'Hydra Reloaded',
            'Outbreak',
            'Pandemic Legion',
            'Suddenly Spaceships.',
            'Verge of Collapse',
            'James 315 Official Fan Club',
        ]
    },
]


npc_lore_questions = [
    {
        'question': 'Who wrote Pax Amarria?',
        'answer': 'Heideran VII',
        'wrong_answers': [
            'Jamyl I',
            'Zaragram II',
            'Damius III',
            'Souro Foiritan',
        ]
    },
    {
        'question': 'Who was the Ray of Matar?',
        'answer': 'Karin Midular',
        'wrong_answers': [
            'Maleatu Shakor',
            'Abja Etbald',
            'Akassa Midular',
            'Ana Utulf',
        ]
    },
    {
        'question': 'What was the Khumaak named after?',
        'answer': 'A person',
        'wrong_answers': [
            'A relic',
            'A ship',
            'A planet',
            'An animal',
        ]
    },
    {
        'question': 'Itzak Barah is...',
        'answer': '... a poet',
        'wrong_answers': [
            '... a soldier',
            '... a priest',
            '... a politician',
            '... a businessman',
        ]
    },
    {
        'question': 'What work did Dr. Adad do?',
        'answer': 'Entertain children',
        'wrong_answers': [
            'Create insorum',
            'Develop cloaking technology',
            'Study sleepers and wormholes',
            'Teach at a university',
        ]
    },
    {
        'question': 'What Caldari political bloc does the Sukuuvestaa megacorp belong to?',
        'answer': 'Practicals',
        'wrong_answers': [
            'Liberals',
            'Profiteers',
            'Draconians',
            'Patriots',
        ]
    },
    {
        'question': 'In what year did the Caldari-Gallente war end?',
        'answer': 'YC 12',
        'wrong_answers': [
            'YC 0',
            'YC 15',
            'YC 50',
            'AD 23148',
        ]
    },
    {
        'question': 'In what year was Concord established?',
        'answer': 'AD 23233',
        'wrong_answers': [
            'YC 0',
            'YC 1',
            'AD 21251',
            'AD 23216',
        ]
    },
    {
        'question': 'In what year AD was New Eden Standard Time introduced?',
        'answer': 'AD 23236',
        'wrong_answers': [
            'AD 21251',
            'AD 23216',
            'YC 1',
            'YC 5',
        ]
    },
]

#############################
# Easy copy and paste formats
#############################
_ignore_me = [
    {
        'question': '',
        'answer': '',
        'wrong_answers': [
            '',
            '',
            '',
            '',
        ]
    },
]
